The errors that can be produced by the crate’s API.
Structs
- Memory
Limit Exceeded Error - An error that occures when rewriter exceedes the memory limit specified in the
MemorySettings. - Parsing
Ambiguity Error - An error that occurs when HTML parser runs into an ambigious state in the
strictmode. - Utf8
Error - Input contained non-UTF-8 byte sequence
Enums
- Attribute
Name Error - An error that occurs when invalid value is provided for the attribute name.
- Comment
Text Error - An error that occurs when invalid value is provided for the HTML comment text.
- Rewriting
Error - A compound error type that can be returned by
writeandendmethods of the rewriter. - Selector
Error - A CSS selector parsing error.
- TagName
Error - An error that occurs when invalid value is provided for the tag name.